getPUMS.2000 <- function(pums.file, PUMA5.filter=NULL){
	# Load the data dictionaries
	load(paste(system.file(package='UScensus'),'/data/housing2000.dict.rda', sep=''))
	load(paste(system.file(package='UScensus'),'/data/person2000.dict.rda', sep=''))
	
	Hvarnames <- unique(housing.dict$VARIABLE)
	Hvarnames <- Hvarnames[-which(Hvarnames=='')]
	# Find the row in which each variable appears for the first time.
	Hfirst.row <- match(Hvarnames,housing.dict$VARIABLE)

	Pvarnames <- unique(person.dict$VARIABLE)
	Pvarnames <- Pvarnames[-which(Pvarnames=='')]
	Pfirst.row <- match(Pvarnames,person.dict$VARIABLE)

	char.vec <- readLines(pums.file, n=-1)
	# Find out what type each row is
	record.type <- substr(char.vec,1,1)
	H.rows <- which(record.type=='H')
	P.rows <- which(record.type=='P')

	if(!is.null(PUMA5.filter)){
		PUMA5 <- substr(char.vec[H.rows],14,18)
		HSERIALNO <- substr(char.vec[H.rows],2,8)
		Hselect.rows <- which(match(PUMA5,PUMA5.filter, nomatch=0)>0)
		PSERIALNO <- substr(char.vec[P.rows],2,8)
		Pselect.rows <- which(match(PSERIALNO,HSERIALNO[Hselect.rows], nomatch=0)>0) 
		writeLines(text=char.vec[H.rows[Hselect.rows]], con=paste('PUMS5_H.TXT',sep=''))
		writeLines(text=char.vec[P.rows[Pselect.rows]], con=paste('PUMS5_P.TXT',sep=''))
		} else {
		writeLines(text=char.vec[H.rows], con=paste('PUMS5_H.TXT',sep=''))
		writeLines(text=char.vec[P.rows], con=paste('PUMS5_P.TXT',sep=''))
		}
	rm(char.vec)

	House.file<-read.fwf(file=paste('PUMS5_H.TXT',sep=''), widths=as.numeric(housing.dict[Hfirst.row,'LEN']), col.names=Hvarnames, colClasses='character')

	for(i in 1:length(Hvarnames)){# For each variable
		# Find the relevant rows in the data dictionary
		dict.rows <- which(housing.dict$VARIABLE==Hvarnames[i])
		# Find if it is a qualitative variable by looking for completely empty HI column
		if(!any(housing.dict[dict.rows,'HI']!='')){
			House.file[,i]<-factor(House.file[,i],levels=housing.dict[dict.rows,'LO'], labels=housing.dict[dict.rows,'VALUE DESCRIPTION'])
			}
		}

	Person.file<-read.fwf(file=paste('PUMS5_P.TXT',sep=''), widths=as.numeric(person.dict[Pfirst.row,'LEN']), col.names=Pvarnames, colClasses='character')

	for(i in 1:length(Pvarnames)){# For each variable
		# Find the relevant rows in the data dictionary
		dict.rows <- which(person.dict$VARIABLE==Pvarnames[i])
		# Find if it is a qualitative variable by looking for completely empty HI column
		if(!any(person.dict[dict.rows,'HI']!='')){
			if(any(person.dict[dict.rows,'LO']!='')){ #Ignore empty LO columns, these are 1% variables anyway
				# There may still be empty LO rows, see RACE2, for example, these may be ignored
				if(any(person.dict[dict.rows,'LO']=='')){dict.rows<- dict.rows[-which(person.dict[dict.rows,'LO']=='')]}
				if(any(person.dict[dict.rows,'VALUE DESCRIPTION']=='Not used')){dict.rows<- dict.rows[-which(person.dict[dict.rows,'VALUE DESCRIPTION']=='Not used')]}
				Person.file[,i]<-factor(Person.file[,i],levels=person.dict[dict.rows,'LO'], labels=person.dict[dict.rows,'VALUE DESCRIPTION'])
				}
			}
		}
	file.remove(paste('PUMS5_P.TXT',sep=''))
	file.remove(paste('PUMS5_H.TXT',sep=''))
	return(list(House.file=House.file,Person.file=Person.file))	}
	
