# SF2000process.R 

# Global variables:
#working.dir <- '/Users/nicholasnagle/Work/microdata/nnagle/SF2000/'
working.dir <- paste(system.file(package="UScensus"),'/data/',sep='')


# 1) Load the data dictionary
data.dict.raw <- read.table(paste(working.dir, '0SF3_table_matrix.csv',sep=''), sep=',', header=FALSE, fill=TRUE, strip.white=TRUE, quote='',colClasses='character')
# 1b) Find out where each uf3 file begins...
file.row <- which(substr(data.dict.raw[,1],1,4)=='File' & substr(data.dict.raw[,1],1,6)!='File I')


# Sanity Check...
# there are 76 uf3 files...
length(file.row) # ought to be 76

file.row <- c(file.row, nrow(data.dict.raw)+1)
data.dict <- NULL

for(file in 1:(length(file.row)-1)){
	this.file <- file # This is the file number
	# this is the row numbers of new tables in this file...
	table.first.rows <- which(!is.na(match(substring(data.dict.raw[file.row[file]:(file.row[file+1]-1),1],1,1),c('P','H'))))
	table.first.rows <- file.row[file]+table.first.rows-1
	# This is the column with table numbers
	table.numbers <- substr(data.dict.raw[table.first.rows,'V1'],1,8)
	# This is the column with table descriptors
	table.names <- data.dict.raw[table.first.rows,'V2']
	
	table.first.rows <- c(table.first.rows, file.row[file+1])
	for(table in 1:(length(table.first.rows)-1)){ #For each table
		# Which rows have columns
		# field.rows <- which(!is.na(match(substr(data.dict.raw[table.first.rows[table]:(table.first.rows[table+1]-1),'V3'],1,1),c('P','H'))))
		# That was the first try, but it also caught Hmong
		field.rows <- which(regexpr('[HP][C01][A-Z0-9]+',data.dict.raw[table.first.rows[table]:(table.first.rows[table+1]-1),'V3'])>0)
		# How many rows?
		num.cols <- length(field.rows)
		# Assemble this table into a temporary matrix
		temp <- data.frame(cbind(	Table = rep(as.character(table.numbers[table]),num.cols),
									TableName=rep(as.character(table.names[table]),num.cols),
									Field = as.character(data.dict.raw[table.first.rows[table]+field.rows-1,'V3']),
									FieldDes = as.character(data.dict.raw[table.first.rows[table]+field.rows-1,'V2']),
									FileNo = rep(file,num.cols),
									column = as.numeric(rep(1,num.cols))))
		# Paste to data dictionary matrix
		data.dict <- rbind(data.dict,temp)
		}
	data.dict$column <- as.numeric(data.dict$column)
	# Data dictionary column is column number from the file.
	data.dict$column[which(data.dict$FileNo==file)] <- seq(from=1, to=length(which(data.dict$FileNo==file)))
	print(file)
}			

data.dict[,1] <- substr(as.character(data.dict[,1]),1,nchar(as.character(data.dict[,1]))-1)
data.dict[,2] <- as.character(data.dict[,2])
data.dict[,3] <- as.character(data.dict[,3])
data.dict[,4] <- as.character(data.dict[,4])
data.dict$FileNo <- sprintf('%05i',data.dict$FileNo)

#3) Load the geo header
geo.raw <- read.table(paste(working.dir, '0SF3_geo_header.csv',sep=''), sep=',', header=FALSE, fill=TRUE, strip.white=TRUE, quote='',colClasses='character')
# Find the good records (as indicated by field names that are capitalized and column numbers that are numbers)
record.rows <- which(regexpr('[A-Z]{2,}',substr(geo.raw$V3,1,2))==1 & regexpr('[0-9]',substr(geo.raw$V5,1,1))==1)

geo.header.dict <- data.frame(cbind( VARNAME = as.character(geo.raw$V3[record.rows]), WIDTH = as.numeric(as.character(geo.raw$V4[record.rows])),POSITION=as.numeric(as.character(geo.raw$V5[record.rows]))))


save(data.dict, geo.header.dict, file=paste(working.dir, 'SF2000_data_dictionary.Rdata', sep=''))


